import 'package:equatable/equatable.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:hydrated_bloc/hydrated_bloc.dart';

import 'package:scrobbles/cubit/settings_global_cubit.dart';
import 'package:scrobbles/models/data/new_tracks.dart';
import 'package:scrobbles/network/scrobbles.dart';

part 'data_new_tracks_state.dart';

class DataNewTracksCubit extends HydratedCubit<DataNewTracksState> {
  DataNewTracksCubit()
      : super(const DataNewTracksState(
          newTracks: null,
          status: '',
        ));

  void setLoading() {
    emit(DataNewTracksState(
      newTracks: state.newTracks,
      status: 'loading',
    ));
  }

  void setValue(NewTracksData? newTracks) {
    emit(DataNewTracksState(
      newTracks: newTracks,
      status: '',
    ));
  }

  void setFailed() {
    emit(DataNewTracksState(
      newTracks: state.newTracks,
      status: 'failed',
    ));
  }

  void setLoaded() {
    emit(DataNewTracksState(
      newTracks: state.newTracks,
      status: '',
    ));
  }

  void update(NewTracksData? newTracks) {
    if ((newTracks != null) && (state.newTracks.toString() != newTracks.toString())) {
      setValue(newTracks);
    } else {
      setLoaded();
    }
  }

  void refresh(BuildContext context) async {
    setLoading();

    GlobalSettingsCubit settings = BlocProvider.of<GlobalSettingsCubit>(context);
    final int count = settings.getNewTracksCount();

    final NewTracksData? data = await ScrobblesApi.fetchNewTracks(count);
    if (data != null) {
      update(data);
    } else {
      setFailed();
    }
  }

  @override
  DataNewTracksState? fromJson(Map<String, dynamic> json) {
    return DataNewTracksState(
      newTracks: NewTracksData.fromJson(json['newTracks']),
      status: '',
    );
  }

  @override
  Map<String, Object?>? toJson(DataNewTracksState state) {
    return <String, Object?>{
      'newTracks': state.newTracks?.toJson(),
      'status': state.status,
    };
  }
}
